@@ -30,7 +30,7 @@
  *            Sven Neumann <sven@convergence.de>
  */
 
-#include <config.h>
+#include "config.h"
 #include "gdk.h"
 
 #include <stdlib.h>
@@ -1898,30 +1898,6 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
 	return (*n_keys > 0);
 }
 
-/**
- * gdk_keymap_translate_keyboard_state:
- * @keymap: a #GdkKeymap, or %NULL to use the default
- * @keycode: a hardware keycode
- * @state: a modifier state 
- * @group: active keyboard group
- * @keyval: return location for keyval
- * @effective_group: return location for effective group
- * @level: return location for level
- * @consumed_modifiers: return location for modifiers that were used to 
- *                      determine the group or level
- *
- * Translates the contents of a #GdkEventKey into a keyval, effective
- * group, and level. Modifiers that affected the translation and
- * are thus unavailable for application use are returned in
- * @consumed_modifiers.  See gdk_keyval_get_keys() for an explanation of
- * groups and levels.  The @effective_group is the group that was
- * actually used for the translation; some keys such as Enter are not
- * affected by the active keyboard group. The @level is derived from
- * @state. For convenience, #GdkEventKey already contains the translated
- * keyval, so this function isn't as useful as you might think.
- * 
- * Return value: %TRUE if there was a keyval bound to the keycode/state/group
- **/
 gboolean
 gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 		guint            keycode,
@@ -1941,32 +1917,41 @@ gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
 
 		if (directfb_keymap[index + i + 2 * group] != GDK_VoidSymbol)
 		{
-			*keyval = directfb_keymap[index + i + 2 * group];
+			if (keyval)
+				*keyval = directfb_keymap[index + i + 2 * group];
 
 			if (group && directfb_keymap[index + i] == *keyval)
 			{
-				*effective_group = 0;
-				*consumed_modifiers = 0;
+				if (effective_group)
+					*effective_group = 0;
+                if(consumed_modifiers)
+				    *consumed_modifiers = 0;
 			}
 			else
 			{
-				*effective_group = group;
-				*consumed_modifiers = GDK_MOD2_MASK;
+				if (effective_group)
+					*effective_group = group;
+                if(consumed_modifiers)
+				    *consumed_modifiers = GDK_MOD2_MASK;
 			}
-
-			*level = i;
+			if (level)
+				*level = i;
 
 			if (i && directfb_keymap[index + 2 * *effective_group] != *keyval)
-				*consumed_modifiers |= GDK_SHIFT_MASK;
+                if(consumed_modifiers)
+				    *consumed_modifiers |= GDK_SHIFT_MASK;
 
 			return TRUE;
 		}
 	}
-
-	*keyval             = 0;
-	*effective_group    = 0;
-	*level              = 0;
-	*consumed_modifiers = 0;
+	if (keyval)
+		*keyval             = 0;
+	if (effective_group)
+		*effective_group    = 0;
+	if (level)
+		*level              = 0;
+    if(consumed_modifiers)
+	    *consumed_modifiers = 0;
 
 	return FALSE;
 }
