@@ -28,7 +28,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <string.h>
 
@@ -184,12 +184,12 @@ gtk_about_dialog_class_init (GtkAboutDialogClass *klass)
 
 
   /**
-   * GtkAboutDialog:name:
+   * GtkAboutDialog:program-name:
    *
    * The name of the program. 
    * If this is not set, it defaults to g_get_application_name().
    *
-   * Since: 2.6
+   * Since: 2.12
    */  
   g_object_class_install_property (object_class,
 				   PROP_NAME,
@@ -700,7 +700,7 @@ gtk_about_dialog_get_property (GObject    *object,
  *
  * Since: 2.6
  *
- * @Deprecated: 2.12: Use gtk_about_dialog_get_program_name() instead.
+ * Deprecated: 2.12: Use gtk_about_dialog_get_program_name() instead.
  **/
 G_CONST_RETURN gchar *
 gtk_about_dialog_get_name (GtkAboutDialog *about)
@@ -765,7 +765,7 @@ update_name_version (GtkAboutDialog *about)
  * 
  * Since: 2.6
  *
- * @Deprecated: 2.12: Use gtk_about_dialog_set_program_name() instead.
+ * Deprecated: 2.12: Use gtk_about_dialog_set_program_name() instead.
  **/
 void
 gtk_about_dialog_set_name (GtkAboutDialog *about, 
@@ -1125,6 +1125,9 @@ gtk_about_dialog_get_website (GtkAboutDialog *about)
  * 
  * Sets the URL to use for the website link.
  *
+ * Note that that the hook functions need to be set up
+ * before calling this function.
+ *
  * Since: 2.6
  **/
 void
@@ -1452,9 +1455,9 @@ gtk_about_dialog_get_translator_credits (GtkAboutDialog *about)
  * of the language which is currently used in the user interface.
  * Using gettext(), a simple way to achieve that is to mark the
  * string for translation:
- * <informalexample><programlisting>
+ * |[
  *  gtk_about_dialog_set_translator_credits (about, _("translator-credits"));
- * </programlisting></informalexample>
+ * ]|
  * It is a good idea to use the customary msgid "translator-credits" for this
  * purpose, since translators will already know the purpose of that msgid, and
  * since #GtkAboutDialog will detect if "translator-credits" is untranslated
@@ -1715,7 +1718,8 @@ credits_key_press_event (GtkWidget      *text_view,
 
   switch (event->keyval)
     {
-      case GDK_Return: 
+      case GDK_Return:
+      case GDK_ISO_Enter:
       case GDK_KP_Enter:
         buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
         gtk_text_buffer_get_iter_at_mark (buffer, &iter, 
@@ -1963,21 +1967,23 @@ add_credits_page (GtkAboutDialog *about,
 	      gchar *link;
 	      const gchar *link_type;
 	      GtkTextTag *tag;
-	      
-	      gtk_text_buffer_insert_at_cursor (buffer, q0, q1 - q0);
-	      gtk_text_buffer_get_end_iter (buffer, &end);
+
+	      if (*q1 == '<')
+		{
+		  gtk_text_buffer_insert_at_cursor (buffer, q0, (q1 - q0) + 1);
+		  gtk_text_buffer_get_end_iter (buffer, &end);
+		  q1++;
+		  link_type = I_("email");
+		}
+	      else
+		{
+		  gtk_text_buffer_insert_at_cursor (buffer, q0, q1 - q0);
+		  gtk_text_buffer_get_end_iter (buffer, &end);
+		  link_type = I_("url");
+		}
 
 	      q0 = q2;
 
-	      if (*q1 == '<') 
-		{
-		  q1++;
-		  q0++;
-		  link_type = I_("email");
-		}
-	      else 
-		link_type = I_("url");
-	      
 	      link = g_strndup (q1, q2 - q1);
 
 	      if (g_slist_find_custom (priv->visited_links, link, (GCompareFunc)strcmp))
