@@ -17,12 +17,13 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include "gtkaccelmap.h"
 
 #include "gtkmarshalers.h"
 #include "gtkwindow.h"  /* in lack of GtkAcceleratable */
+#include "gtkintl.h" 
 #include "gtkalias.h"
 
 #include <glib/gstdio.h>
@@ -36,7 +37,6 @@
 #ifdef G_OS_WIN32
 #include <io.h>
 #endif
-#include <errno.h>
 
 
 /* --- structures --- */
@@ -56,8 +56,8 @@ typedef struct {
   guint        accel_mods;
   guint	       std_accel_key;
   guint	       std_accel_mods;
-  guint        changed : 1;
-  guint        lock_count;
+  guint        changed    :  1;
+  guint        lock_count : 15;
   GSList      *groups;
 } AccelEntry;
 
@@ -149,6 +149,10 @@ _gtk_accel_path_is_valid (const gchar *accel_path)
  * e.g. "File/Save As", "Image/View/Zoom" or "Edit/Select All".
  * So a full valid accelerator path may look like:
  * "&lt;Gimp-Toolbox&gt;/File/Dialogs/Tool Options...".
+ * 
+ * Note that @accel_path string will be stored in a #GQuark. Therefore, if you
+ * pass a static string, you can save some memory by interning it first with 
+ * g_intern_static_string().
  */
 void
 gtk_accel_map_add_entry (const gchar    *accel_path,
@@ -178,8 +182,8 @@ gtk_accel_map_add_entry (const gchar    *accel_path,
     }
   else
     {
-      entry = g_new0 (AccelEntry, 1);
-      entry->accel_path = g_quark_to_string (g_quark_from_string (accel_path));
+      entry = g_slice_new0 (AccelEntry);
+      entry->accel_path = g_intern_string (accel_path);
       entry->std_accel_key = accel_key;
       entry->std_accel_mods = accel_mods;
       entry->accel_key = accel_key;
@@ -386,8 +390,6 @@ internal_change_entry (const gchar    *accel_path,
   
   if (change_accel && !simulate)
     {
-      guint old_accel_key, old_accel_mods;
-      
       /* ref accel groups */
       for (slist = group_list; slist; slist = slist->next)
 	g_object_ref (slist->data);
@@ -397,8 +399,6 @@ internal_change_entry (const gchar    *accel_path,
 	internal_change_entry (g_quark_to_string (GPOINTER_TO_UINT (slist->data)), 0, 0, FALSE, FALSE);
 
       /* 9) install new accelerator */
-      old_accel_key = entry->accel_key;
-      old_accel_mods = entry->accel_mods;
       entry->accel_key = accel_key;
       entry->accel_mods = accel_mods;
       entry->changed = TRUE;
@@ -433,6 +433,10 @@ internal_change_entry (const gchar    *accel_path,
  * conflicts. A change will only occur if all conflicts could be resolved (which
  * might not be the case if conflicting accelerators are locked). Successful
  * changes are indicated by a %TRUE return value.
+ * 
+ * Note that @accel_path string will be stored in a #GQuark. Therefore, if you
+ * pass a static string, you can save some memory by interning it first with 
+ * g_intern_static_string().
  */
 gboolean
 gtk_accel_map_change_entry (const gchar    *accel_path,
@@ -545,7 +549,7 @@ gtk_accel_map_load_scanner (GScanner *scanner)
   gchar *cpair_comment_single;
   gpointer saved_symbol;
   
-  g_return_if_fail (scanner != 0);
+  g_return_if_fail (scanner != NULL);
 
   /* configure scanner */
   skip_comment_single = scanner->config->skip_comment_single;
@@ -934,7 +938,7 @@ gtk_accel_map_unlock_path (const gchar *accel_path)
   entry->lock_count -= 1;  
 }
 
-G_DEFINE_TYPE (GtkAccelMap, gtk_accel_map, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GtkAccelMap, gtk_accel_map, G_TYPE_OBJECT)
 
 static void
 gtk_accel_map_class_init (GtkAccelMapClass *accel_map_class)
@@ -953,7 +957,7 @@ gtk_accel_map_class_init (GtkAccelMapClass *accel_map_class)
    *
    * Since: 2.4
    */
-  accel_map_signals[CHANGED] = g_signal_new ("changed",
+  accel_map_signals[CHANGED] = g_signal_new (I_("changed"),
 					     G_TYPE_FROM_CLASS (accel_map_class),
 					     G_SIGNAL_DETAILED|G_SIGNAL_RUN_LAST,
 					     0,
