@@ -20,7 +20,7 @@
  * Global clipboard abstraction. 
  */
 
-#include <config.h>
+#include "config.h"
 #include <string.h>
 
 #include "gtkclipboard.h"
@@ -50,6 +50,7 @@ typedef struct _RequestContentsInfo RequestContentsInfo;
 typedef struct _RequestTextInfo RequestTextInfo;
 typedef struct _RequestRichTextInfo RequestRichTextInfo;
 typedef struct _RequestImageInfo RequestImageInfo;
+typedef struct _RequestURIInfo RequestURIInfo;
 typedef struct _RequestTargetsInfo RequestTargetsInfo;
 
 struct _GtkClipboard 
@@ -114,6 +115,12 @@ struct _RequestImageInfo
   gpointer user_data;
 };
 
+struct _RequestURIInfo
+{
+  GtkClipboardURIReceivedFunc callback;
+  gpointer user_data;
+};
+
 struct _RequestTargetsInfo
 {
   GtkClipboardTargetsReceivedFunc callback;
@@ -167,6 +174,17 @@ gtk_clipboard_class_init (GtkClipboardClass *class)
 
   class->owner_change = gtk_clipboard_owner_change;
 
+  /**
+   * GtkClipboard::owner-change:
+   * @clipboard: the #GtkClipboard on which the signal is emitted
+   * @event: the @GdkEventOwnerChange event 
+   *
+   * The ::owner-change signal is emitted when GTK+ receives an
+   * event that indicates that the ownership of the selection 
+   * associated with @clipboard has changed.
+   *
+   * Since: 2.6
+   */ 
   clipboard_signals[OWNER_CHANGE] =
     g_signal_new (I_("owner_change"),
 		  G_TYPE_FROM_CLASS (gobject_class),
@@ -182,26 +200,31 @@ static void
 gtk_clipboard_finalize (GObject *object)
 {
   GtkClipboard *clipboard;
-  GtkWidget *clipboard_widget;
-  GSList *clipboards;
+  GtkWidget *clipboard_widget = NULL;
+  GSList *clipboards = NULL;
 
   clipboard = GTK_CLIPBOARD (object);
 
-  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
-  if (g_slist_index (clipboards, clipboard) >= 0)
-    g_warning ("GtkClipboard prematurely finalized");
+  if (clipboard->display)
+    {
+      clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
 
-  /*  don't use get_clipboard_widget() here because it would create the
-   *  widget if it doesn't exist.
-   */
-  clipboard_widget = g_object_get_data (G_OBJECT (clipboard->display),
-                                        "gtk-clipboard-widget");
+      if (g_slist_index (clipboards, clipboard) >= 0)
+        g_warning ("GtkClipboard prematurely finalized");
+
+      clipboards = g_slist_remove (clipboards, clipboard);
+
+      g_object_set_data (G_OBJECT (clipboard->display), "gtk-clipboard-list", 
+                         clipboards);
+
+      /* don't use get_clipboard_widget() here because it would create the
+       * widget if it doesn't exist.
+       */
+      clipboard_widget = g_object_get_data (G_OBJECT (clipboard->display),
+                                            "gtk-clipboard-widget");
+    }
 
   clipboard_unset (clipboard);
-  
-  clipboards = g_object_get_data (G_OBJECT (clipboard->display), "gtk-clipboard-list");
-  clipboards = g_slist_remove (clipboards, clipboard);
-  g_object_set_data (G_OBJECT (clipboard->display), I_("gtk-clipboard-list"), clipboards);
 
   if (clipboard->store_loop && g_main_loop_is_running (clipboard->store_loop))
     g_main_loop_quit (clipboard->store_loop);
@@ -728,9 +751,8 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 			gint          len)
 {
   GtkTargetList *list;
-  GList *l;
   GtkTargetEntry *targets;
-  gint n_targets, i;
+  gint n_targets;
 
   g_return_if_fail (clipboard != NULL);
   g_return_if_fail (text != NULL);
@@ -738,13 +760,7 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
   list = gtk_target_list_new (NULL, 0);
   gtk_target_list_add_text_targets (list, 0);
 
-  n_targets = g_list_length (list->list);
-  targets = g_new0 (GtkTargetEntry, n_targets);
-  for (l = list->list, i = 0; l; l = l->next, i++)
-    {
-      GtkTargetPair *pair = (GtkTargetPair *)l->data;
-      targets[i].target = gdk_atom_name (pair->target);
-    }
+  targets = gtk_target_table_new_from_list (list, &n_targets);
   
   if (len < 0)
     len = strlen (text);
@@ -755,9 +771,7 @@ gtk_clipboard_set_text (GtkClipboard *clipboard,
 			       g_strndup (text, len));
   gtk_clipboard_set_can_store (clipboard, NULL, 0);
 
-  for (i = 0; i < n_targets; i++)
-    g_free (targets[i].target);
-  g_free (targets);
+  gtk_target_table_free (targets, n_targets);
   gtk_target_list_unref (list);
 }
 
@@ -794,9 +808,8 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
 			  GdkPixbuf    *pixbuf)
 {
   GtkTargetList *list;
-  GList *l;
   GtkTargetEntry *targets;
-  gint n_targets, i;
+  gint n_targets;
 
   g_return_if_fail (clipboard != NULL);
   g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
@@ -804,13 +817,7 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
   list = gtk_target_list_new (NULL, 0);
   gtk_target_list_add_image_targets (list, 0, TRUE);
 
-  n_targets = g_list_length (list->list);
-  targets = g_new0 (GtkTargetEntry, n_targets);
-  for (l = list->list, i = 0; l; l = l->next, i++)
-    {
-      GtkTargetPair *pair = (GtkTargetPair *)l->data;
-      targets[i].target = gdk_atom_name (pair->target);
-    }
+  targets = gtk_target_table_new_from_list (list, &n_targets);
 
   gtk_clipboard_set_with_data (clipboard, 
 			       targets, n_targets,
@@ -818,9 +825,7 @@ gtk_clipboard_set_image (GtkClipboard *clipboard,
 			       g_object_ref (pixbuf));
   gtk_clipboard_set_can_store (clipboard, NULL, 0);
 
-  for (i = 0; i < n_targets; i++)
-    g_free (targets[i].target);
-  g_free (targets);
+  gtk_target_table_free (targets, n_targets);
   gtk_target_list_unref (list);
 }
 
@@ -1141,6 +1146,58 @@ gtk_clipboard_request_image (GtkClipboard                  *clipboard,
 				  info);
 }
 
+static void 
+request_uris_received_func (GtkClipboard     *clipboard,
+			    GtkSelectionData *selection_data,
+			    gpointer          data)
+{
+  RequestURIInfo *info = data;
+  gchar **uris;
+
+  uris = gtk_selection_data_get_uris (selection_data);
+  info->callback (clipboard, uris, info->user_data);
+  g_strfreev (uris);
+
+  g_slice_free (RequestURIInfo, info);
+}
+
+/**
+ * gtk_clipboard_request_uris:
+ * @clipboard: a #GtkClipboard
+ * @callback:  a function to call when the URIs are received,
+ *             or the retrieval fails. (It will always be called
+ *             one way or the other.)
+ * @user_data: user data to pass to @callback.
+ * 
+ * Requests the contents of the clipboard as URIs. When the URIs are
+ * later received @callback will be called.
+ *
+ * The @uris parameter to @callback will contain the resulting array of
+ * URIs if the request succeeded, or %NULL if it failed. This could happen
+ * for various reasons, in particular if the clipboard was empty or if the
+ * contents of the clipboard could not be converted into URI form.
+ *
+ * Since: 2.14
+ **/
+void 
+gtk_clipboard_request_uris (GtkClipboard                *clipboard,
+			    GtkClipboardURIReceivedFunc  callback,
+			    gpointer                     user_data)
+{
+  RequestURIInfo *info;
+  
+  g_return_if_fail (clipboard != NULL);
+  g_return_if_fail (callback != NULL);
+  
+  info = g_slice_new (RequestURIInfo);
+  info->callback = callback;
+  info->user_data = user_data;
+
+  gtk_clipboard_request_contents (clipboard, gdk_atom_intern_static_string ("text/uri-list"),
+				  request_uris_received_func,
+				  info);
+}
+
 static void 
 request_targets_received_func (GtkClipboard     *clipboard,
 			       GtkSelectionData *selection_data,
@@ -1445,6 +1502,60 @@ gtk_clipboard_wait_for_image (GtkClipboard *clipboard)
   return results.data;
 }
 
+static void 
+clipboard_uris_received_func (GtkClipboard *clipboard,
+			      gchar       **uris,
+			      gpointer      data)
+{
+  WaitResults *results = data;
+
+  results->data = g_strdupv (uris);
+  g_main_loop_quit (results->loop);
+}
+
+/**
+ * gtk_clipboard_wait_for_uris:
+ * @clipboard: a #GtkClipboard
+ * 
+ * Requests the contents of the clipboard as URIs. This function waits
+ * for the data to be received using the main loop, so events,
+ * timeouts, etc, may be dispatched during the wait.
+ * 
+ * Return value: a newly-allocated %NULL-terminated array of strings which must
+ *               be freed with g_strfreev(), or %NULL if
+ *               retrieving the selection data failed. (This 
+ *               could happen for various reasons, in particular 
+ *               if the clipboard was empty or if the contents of 
+ *               the clipboard could not be converted into URI form.)
+ *
+ * Since: 2.14
+ **/
+gchar **
+gtk_clipboard_wait_for_uris (GtkClipboard *clipboard)
+{
+  WaitResults results;
+
+  g_return_val_if_fail (clipboard != NULL, NULL);
+  
+  results.data = NULL;
+  results.loop = g_main_loop_new (NULL, TRUE);
+
+  gtk_clipboard_request_uris (clipboard,
+			      clipboard_uris_received_func,
+			      &results);
+
+  if (g_main_loop_is_running (results.loop))
+    {
+      GDK_THREADS_LEAVE ();
+      g_main_loop_run (results.loop);
+      GDK_THREADS_ENTER ();
+    }
+
+  g_main_loop_unref (results.loop);
+
+  return results.data;
+}
+
 /**
  * gtk_clipboard_get_display:
  * @clipboard: a #GtkClipboard
@@ -1569,6 +1680,41 @@ gtk_clipboard_wait_is_image_available (GtkClipboard *clipboard)
   return result;
 }
 
+/**
+ * gtk_clipboard_wait_is_uris_available:
+ * @clipboard: a #GtkClipboard
+ * 
+ * Test to see if there is a list of URIs available to be pasted
+ * This is done by requesting the TARGETS atom and checking
+ * if it contains the URI targets. This function
+ * waits for the data to be received using the main loop, so events, 
+ * timeouts, etc, may be dispatched during the wait.
+ *
+ * This function is a little faster than calling
+ * gtk_clipboard_wait_for_uris() since it doesn't need to retrieve
+ * the actual URI data.
+ * 
+ * Return value: %TRUE is there is an URI list available, %FALSE otherwise.
+ *
+ * Since: 2.14
+ **/
+gboolean
+gtk_clipboard_wait_is_uris_available (GtkClipboard *clipboard)
+{
+  GtkSelectionData *data;
+  gboolean result = FALSE;
+
+  data = gtk_clipboard_wait_for_contents (clipboard, 
+					  gdk_atom_intern_static_string ("TARGETS"));
+  if (data)
+    {
+      result = gtk_selection_data_targets_include_uri (data);
+      gtk_selection_data_free (data);
+    }
+
+  return result;
+}
+
 /**
  * gtk_clipboard_wait_for_targets
  * @clipboard: a #GtkClipboard
@@ -1744,7 +1890,7 @@ gtk_clipboard_wait_is_target_available (GtkClipboard *clipboard,
  * _gtk_clipboard_handle_event:
  * @event: a owner change event
  * 
- * Emits the ::owner_change signal on the appropriate @clipboard.
+ * Emits the #GtkClipboard::owner-change signal on the appropriate @clipboard.
  *
  * Since: 2.6
  **/
@@ -1862,6 +2008,8 @@ gtk_clipboard_store (GtkClipboard *clipboard)
   if (!gdk_display_supports_clipboard_persistence (clipboard->display))
     return;
 
+  g_object_ref (clipboard);
+
   clipboard_widget = get_clipboard_widget (clipboard->display);
   clipboard->notify_signal_id = g_signal_connect (clipboard_widget, "selection_notify_event",
 						  G_CALLBACK (gtk_clipboard_selection_notify), clipboard);
@@ -1893,6 +2041,8 @@ gtk_clipboard_store (GtkClipboard *clipboard)
   clipboard->notify_signal_id = 0;
   
   clipboard->storing_selection = FALSE;
+
+  g_object_unref (clipboard);
 }
 
 /* Stores all clipboard selections on all displays, called from
