@@ -19,9 +19,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
+
+#undef GTK_DISABLE_DEPRECATED /* GtkTooltips */
+
 #include "gtkmenutoolbutton.h"
-#include "gtkintl.h"
 #include "gtktogglebutton.h"
 #include "gtkarrow.h"
 #include "gtkhbox.h"
@@ -29,6 +31,7 @@
 #include "gtkmenu.h"
 #include "gtkmain.h"
 #include "gtkprivate.h"
+#include "gtkintl.h"
 #include "gtkalias.h"
 
 
@@ -57,30 +60,13 @@ enum
 enum
 {
   PROP_0,
-  PROP_MENU,
-  LAST_PROP
+  PROP_MENU
 };
 
 static gint signals[LAST_SIGNAL];
 
 G_DEFINE_TYPE (GtkMenuToolButton, gtk_menu_tool_button, GTK_TYPE_TOOL_BUTTON)
 
-static gboolean
-gtk_menu_tool_button_set_tooltip (GtkToolItem *tool_item,
-                                  GtkTooltips *tooltips,
-                                  const char  *tip_text,
-                                  const char  *tip_private)
-{
-  GtkMenuToolButton *button;
-
-  g_return_val_if_fail (GTK_IS_MENU_TOOL_BUTTON (tool_item), FALSE);
-
-  button = GTK_MENU_TOOL_BUTTON (tool_item);
-  gtk_tooltips_set_tip (tooltips, button->priv->button, tip_text, tip_private);
-
-  return TRUE;
-}
-
 static void
 gtk_menu_tool_button_construct_contents (GtkMenuToolButton *button)
 {
@@ -122,6 +108,17 @@ gtk_menu_tool_button_construct_contents (GtkMenuToolButton *button)
 
   if (priv->box)
     {
+      gchar *tmp;
+
+      /* Transfer a possible tooltip to the new box */
+      g_object_get (priv->box, "tooltip-markup", &tmp, NULL);
+
+      if (tmp)
+        {
+	  g_object_set (box, "tooltip-markup", tmp, NULL);
+	  g_free (tmp);
+	}
+
       /* Note: we are not destroying the button and the arrow_button
        * here because they were removed from their container above
        */
@@ -218,7 +215,6 @@ gtk_menu_tool_button_class_init (GtkMenuToolButtonClass *klass)
   object_class->get_property = gtk_menu_tool_button_get_property;
   gtk_object_class->destroy = gtk_menu_tool_button_destroy;
   widget_class->state_changed = gtk_menu_tool_button_state_changed;
-  toolitem_class->set_tooltip = gtk_menu_tool_button_set_tooltip;
   toolitem_class->toolbar_reconfigured = gtk_menu_tool_button_toolbar_reconfigured;
 
   signals[SHOW_MENU] =
