@@ -24,19 +24,14 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#if HAVE_CONFIG_H
-#  include <config.h>
-#  if STDC_HEADERS
-#    include <string.h>
-#    include <stdio.h>
-#  endif
-#else
-#  include <stdio.h>
-#endif
+#include "config.h"
+
+#include <string.h>
 
-#include "gtkalias.h"
 #include "gtkprogressbar.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkalias.h"
 
 
 #define MIN_HORIZONTAL_BAR_WIDTH   150
@@ -44,7 +39,6 @@
 #define MIN_VERTICAL_BAR_WIDTH     22
 #define MIN_VERTICAL_BAR_HEIGHT    80
 #define MAX_TEXT_LENGTH            80
-#define TEXT_SPACING               2
 
 enum {
   PROP_0,
@@ -54,6 +48,7 @@ enum {
   PROP_PULSE_STEP,
   PROP_ORIENTATION,
   PROP_TEXT,
+  PROP_ELLIPSIZE,
   
   /* Deprecated args */
   PROP_ADJUSTMENT,
@@ -63,8 +58,6 @@ enum {
   PROP_DISCRETE_BLOCKS
 };
 
-static void gtk_progress_bar_class_init    (GtkProgressBarClass *klass);
-static void gtk_progress_bar_init          (GtkProgressBar      *pbar);
 static void gtk_progress_bar_set_property  (GObject             *object,
 					    guint                prop_id,
 					    const GValue        *value,
@@ -73,8 +66,12 @@ static void gtk_progress_bar_get_property  (GObject             *object,
 					    guint                prop_id,
 					    GValue              *value,
 					    GParamSpec          *pspec);
+static gboolean gtk_progress_bar_expose    (GtkWidget           *widget,
+					    GdkEventExpose      *event);
 static void gtk_progress_bar_size_request  (GtkWidget           *widget,
 					    GtkRequisition      *requisition);
+static void gtk_progress_bar_style_set     (GtkWidget           *widget,
+					    GtkStyle            *previous);
 static void gtk_progress_bar_real_update   (GtkProgress         *progress);
 static void gtk_progress_bar_paint         (GtkProgress         *progress);
 static void gtk_progress_bar_act_mode_enter (GtkProgress        *progress);
@@ -89,33 +86,7 @@ static void gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
 							   guint           blocks);
 
 
-GType
-gtk_progress_bar_get_type (void)
-{
-  static GType progress_bar_type = 0;
-
-  if (!progress_bar_type)
-    {
-      static const GTypeInfo progress_bar_info =
-      {
-	sizeof (GtkProgressBarClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_progress_bar_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkProgressBar),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_progress_bar_init,
-      };
-
-      progress_bar_type =
-	g_type_register_static (GTK_TYPE_PROGRESS, "GtkProgressBar",
-				&progress_bar_info, 0);
-    }
-
-  return progress_bar_type;
-}
+G_DEFINE_TYPE (GtkProgressBar, gtk_progress_bar, GTK_TYPE_PROGRESS)
 
 static void
 gtk_progress_bar_class_init (GtkProgressBarClass *class)
@@ -131,7 +102,9 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
   gobject_class->set_property = gtk_progress_bar_set_property;
   gobject_class->get_property = gtk_progress_bar_get_property;
   
+  widget_class->expose_event = gtk_progress_bar_expose;
   widget_class->size_request = gtk_progress_bar_size_request;
+  widget_class->style_set = gtk_progress_bar_style_set;
 
   progress_class->paint = gtk_progress_bar_paint;
   progress_class->update = gtk_progress_bar_real_update;
@@ -143,7 +116,7 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
                                                         P_("Adjustment"),
                                                         P_("The GtkAdjustment connected to the progress bar (Deprecated)"),
                                                         GTK_TYPE_ADJUSTMENT,
-                                                        G_PARAM_READWRITE));
+                                                        GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ORIENTATION,
@@ -152,75 +125,100 @@ gtk_progress_bar_class_init (GtkProgressBarClass *class)
 						      P_("Orientation and growth direction of the progress bar"),
 						      GTK_TYPE_PROGRESS_BAR_ORIENTATION,
 						      GTK_PROGRESS_LEFT_TO_RIGHT,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_BAR_STYLE,
-                                   g_param_spec_enum ("bar_style",
+                                   g_param_spec_enum ("bar-style",
 						      P_("Bar style"),
 						      P_("Specifies the visual style of the bar in percentage mode (Deprecated)"),
 						      GTK_TYPE_PROGRESS_BAR_STYLE,
 						      GTK_PROGRESS_CONTINUOUS,
-						      G_PARAM_READWRITE));
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_STEP,
-                                   g_param_spec_uint ("activity_step",
+                                   g_param_spec_uint ("activity-step",
 						      P_("Activity Step"),
 						      P_("The increment used for each iteration in activity mode (Deprecated)"),
-						      -G_MAXUINT,
-						      G_MAXUINT,
-						      3,
-						      G_PARAM_READWRITE));
+						      0, G_MAXUINT, 3,
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_ACTIVITY_BLOCKS,
-                                   g_param_spec_uint ("activity_blocks",
+                                   g_param_spec_uint ("activity-blocks",
 						      P_("Activity Blocks"),
 						      P_("The number of blocks which can fit in the progress bar area in activity mode (Deprecated)"),
-						      2,
-						      G_MAXUINT,
-						      5,
-						      G_PARAM_READWRITE));
+						      2, G_MAXUINT, 5,
+						      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_DISCRETE_BLOCKS,
-                                   g_param_spec_uint ("discrete_blocks",
+                                   g_param_spec_uint ("discrete-blocks",
 						      P_("Discrete Blocks"),
 						      P_("The number of discrete blocks in a progress bar (when shown in the discrete style)"),
-						      2,
-						      G_MAXUINT,
-						      10,
-						      G_PARAM_READWRITE));
+						      2, G_MAXUINT, 10,
+						      GTK_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
 				   PROP_FRACTION,
 				   g_param_spec_double ("fraction",
 							P_("Fraction"),
 							P_("The fraction of total work that has been completed"),
-							0.0,
-							1.0,
-							0.0,
-							G_PARAM_READWRITE));  
+							0.0, 1.0, 0.0,
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_PULSE_STEP,
-				   g_param_spec_double ("pulse_step",
+				   g_param_spec_double ("pulse-step",
 							P_("Pulse Step"),
 							P_("The fraction of total progress to move the bouncing block when pulsed"),
-							0.0,
-							1.0,
-							0.1,
-							G_PARAM_READWRITE));  
+							0.0, 1.0, 0.1,
+							GTK_PARAM_READWRITE));  
   
   g_object_class_install_property (gobject_class,
 				   PROP_TEXT,
 				   g_param_spec_string ("text",
 							P_("Text"),
 							P_("Text to be displayed in the progress bar"),
-							"%P %%",
-							G_PARAM_READWRITE));
+							NULL,
+							GTK_PARAM_READWRITE));
 
+  /**
+   * GtkProgressBar:ellipsize:
+   *
+   * The preferred place to ellipsize the string, if the progressbar does 
+   * not have enough room to display the entire string, specified as a 
+   * #PangoEllisizeMode. 
+   *
+   * Note that setting this property to a value other than 
+   * %PANGO_ELLIPSIZE_NONE has the side-effect that the progressbar requests 
+   * only enough space to display the ellipsis "...". Another means to set a 
+   * progressbar's width is gtk_widget_set_size_request().
+   *
+   * Since: 2.6
+   */
+  g_object_class_install_property (gobject_class,
+				   PROP_ELLIPSIZE,
+                                   g_param_spec_enum ("ellipsize",
+                                                      P_("Ellipsize"),
+                                                      P_("The preferred place to ellipsize the string, if the progress bar "
+                                                         "does not have enough room to display the entire string, if at all."),
+						      PANGO_TYPE_ELLIPSIZE_MODE,
+						      PANGO_ELLIPSIZE_NONE,
+                                                      GTK_PARAM_READWRITE));
+  gtk_widget_class_install_style_property (widget_class,
+                                           g_param_spec_int ("xspacing",
+                                                             P_("XSpacing"),
+                                                             P_("Extra spacing applied to the width of a progress bar."),
+                                                             0, G_MAXINT, 7,
+                                                             G_PARAM_READWRITE));
+  gtk_widget_class_install_style_property (widget_class,
+                                           g_param_spec_int ("yspacing",
+                                                             "YSpacing",
+                                                             "Extra spacing applied to the height of a progress bar.",
+                                                             0, G_MAXINT, 7,
+                                                             G_PARAM_READWRITE));
 }
 
 static void
@@ -235,6 +233,7 @@ gtk_progress_bar_init (GtkProgressBar *pbar)
   pbar->activity_dir = 1;
   pbar->activity_step = 3;
   pbar->activity_blocks = 5;
+  pbar->ellipsize = PANGO_ELLIPSIZE_NONE;
 }
 
 static void
@@ -277,6 +276,9 @@ gtk_progress_bar_set_property (GObject      *object,
     case PROP_TEXT:
       gtk_progress_bar_set_text (pbar, g_value_get_string (value));
       break;
+    case PROP_ELLIPSIZE:
+      gtk_progress_bar_set_ellipsize (pbar, g_value_get_enum (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -322,6 +324,9 @@ gtk_progress_bar_get_property (GObject      *object,
     case PROP_TEXT:
       g_value_set_string (value, gtk_progress_bar_get_text (pbar));
       break;
+    case PROP_ELLIPSIZE:
+      g_value_set_enum (value, pbar->ellipsize);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -333,7 +338,7 @@ gtk_progress_bar_new (void)
 {
   GtkWidget *pbar;
 
-  pbar = gtk_widget_new (GTK_TYPE_PROGRESS_BAR, NULL);
+  pbar = g_object_new (GTK_TYPE_PROGRESS_BAR, NULL);
 
   return pbar;
 }
@@ -345,7 +350,7 @@ gtk_progress_bar_new_with_adjustment (GtkAdjustment *adjustment)
 
   g_return_val_if_fail (GTK_IS_ADJUSTMENT (adjustment), NULL);
 
-  pbar = gtk_widget_new (GTK_TYPE_PROGRESS_BAR,
+  pbar = g_object_new (GTK_TYPE_PROGRESS_BAR,
 			 "adjustment", adjustment,
 			 NULL);
 
@@ -434,7 +439,7 @@ gtk_progress_bar_real_update (GtkProgress *progress)
 		}
 	    }
 	}
-      gtk_progress_bar_paint (progress);
+      pbar->dirty = TRUE;
       gtk_widget_queue_draw (GTK_WIDGET (progress));
     }
   else
@@ -447,12 +452,28 @@ gtk_progress_bar_real_update (GtkProgress *progress)
       if (pbar->in_block != in_block)
 	{
 	  pbar->in_block = in_block;
-	  gtk_progress_bar_paint (progress);
+	  pbar->dirty = TRUE;
 	  gtk_widget_queue_draw (GTK_WIDGET (progress));
 	}
     }
 }
 
+static gboolean
+gtk_progress_bar_expose (GtkWidget      *widget,
+		     GdkEventExpose *event)
+{
+  GtkProgressBar *pbar;
+
+  g_return_val_if_fail (GTK_IS_PROGRESS_BAR (widget), FALSE);
+
+  pbar = GTK_PROGRESS_BAR (widget);
+
+  if (GTK_WIDGET_DRAWABLE (widget) && pbar->dirty)
+    gtk_progress_bar_paint (GTK_PROGRESS (pbar));
+
+  return GTK_WIDGET_CLASS (gtk_progress_bar_parent_class)->expose_event (widget, event);
+}
+
 static void
 gtk_progress_bar_size_request (GtkWidget      *widget,
 			       GtkRequisition *requisition)
@@ -462,13 +483,23 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   gchar *buf;
   PangoRectangle logical_rect;
   PangoLayout *layout;
+  gint width, height;
+  gint xspacing, yspacing;
 
   g_return_if_fail (GTK_IS_PROGRESS_BAR (widget));
   g_return_if_fail (requisition != NULL);
 
+  gtk_widget_style_get (widget,
+                        "xspacing", &xspacing,
+                        "yspacing", &yspacing,
+                        NULL);
+
   progress = GTK_PROGRESS (widget);
   pbar = GTK_PROGRESS_BAR (widget);
 
+  width = 2 * widget->style->xthickness + xspacing;
+  height = 2 * widget->style->ythickness + yspacing;
+
   if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
     {
       if (!progress->adjustment)
@@ -477,8 +508,29 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
       buf = gtk_progress_get_text_from_value (progress, progress->adjustment->upper);
 
       layout = gtk_widget_create_pango_layout (widget, buf);
+
       pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 	  
+      if (pbar->ellipsize)
+	{
+	  PangoContext *context;
+	  PangoFontMetrics *metrics;
+	  gint char_width;
+	  
+	  /* The minimum size for ellipsized text is ~ 3 chars */
+	  context = pango_layout_get_context (layout);
+	  metrics = pango_context_get_metrics (context, widget->style->font_desc, pango_context_get_language (context));
+	  
+	  char_width = pango_font_metrics_get_approximate_char_width (metrics);
+	  pango_font_metrics_unref (metrics);
+	  
+	  width += PANGO_PIXELS (char_width) * 3;
+	}
+      else
+	width += logical_rect.width;
+      
+      height += logical_rect.height;
+
       g_object_unref (layout);
       g_free (buf);
     }
@@ -486,48 +538,32 @@ gtk_progress_bar_size_request (GtkWidget      *widget,
   if (pbar->orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       pbar->orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{
-	  requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
-	}
-      else
-	{
-	  requisition->width = MIN_HORIZONTAL_BAR_WIDTH;
-	  requisition->height = MIN_HORIZONTAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_HORIZONTAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_HORIZONTAL_BAR_HEIGHT, height);
     }
   else
     {
-      if (progress->show_text && pbar->bar_style != GTK_PROGRESS_DISCRETE)
-	{	  
-	  requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH,
-				    2 * widget->style->xthickness + 3 +
-				    logical_rect.width + 2 * TEXT_SPACING);
-
-	  requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT,
-				     2 * widget->style->ythickness + 3 +
-				     logical_rect.height + 2 * TEXT_SPACING);
-	}
-      else
-	{
-	  requisition->width = MIN_VERTICAL_BAR_WIDTH;
-	  requisition->height = MIN_VERTICAL_BAR_HEIGHT;
-	}
+      requisition->width = MAX (MIN_VERTICAL_BAR_WIDTH, width);
+      requisition->height = MAX (MIN_VERTICAL_BAR_HEIGHT, height);
     }
 }
 
+static void
+gtk_progress_bar_style_set (GtkWidget      *widget,
+    GtkStyle *previous)
+{
+  GtkProgressBar *pbar = GTK_PROGRESS_BAR (widget);
+
+  pbar->dirty = TRUE;
+
+  GTK_WIDGET_CLASS (gtk_progress_bar_parent_class)->style_set (widget, previous);
+}
+
 static void
 gtk_progress_bar_act_mode_enter (GtkProgress *progress)
 {
   GtkProgressBar *pbar;
   GtkWidget *widget;
-  gint size;
   GtkProgressBarOrientation orientation;
 
   pbar = GTK_PROGRESS_BAR (progress);
@@ -547,8 +583,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
   if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT ||
       orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
     {
-      size = MAX (2, widget->allocation.width / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_LEFT_TO_RIGHT)
 	{
 	  pbar->activity_pos = widget->style->xthickness;
@@ -564,8 +598,6 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
     }
   else
     {
-      size = MAX (2, widget->allocation.height / pbar->activity_blocks);
-
       if (orientation == GTK_PROGRESS_TOP_TO_BOTTOM)
 	{
 	  pbar->activity_pos = widget->style->ythickness;
@@ -582,29 +614,51 @@ gtk_progress_bar_act_mode_enter (GtkProgress *progress)
 }
 
 static void
-gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
-				 GtkProgressBarOrientation  orientation)
+gtk_progress_bar_get_activity (GtkProgressBar            *pbar,
+			       GtkProgressBarOrientation  orientation,
+			       gint                      *offset,
+			       gint                      *amount)
 {
   GtkWidget *widget = GTK_WIDGET (pbar);
-  GtkProgress *progress = GTK_PROGRESS (pbar);
-  gint x, y, w, h;
+
+  *offset = pbar->activity_pos;
 
   switch (orientation)
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      x = pbar->activity_pos;
-      y = widget->style->ythickness;
-      w = MAX (2, widget->allocation.width / pbar->activity_blocks);
-      h = widget->allocation.height - 2 * widget->style->ythickness;
+      *amount = MAX (2, widget->allocation.width / pbar->activity_blocks);
       break;
 
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      x = widget->style->xthickness;
-      y = pbar->activity_pos;
-      w = widget->allocation.width - 2 * widget->style->xthickness;
-      h = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      *amount = MAX (2, widget->allocation.height / pbar->activity_blocks);
+      break;
+    }
+}
+
+static void
+gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
+				 GtkProgressBarOrientation  orientation)
+{
+  GtkWidget *widget = GTK_WIDGET (pbar);
+  GtkProgress *progress = GTK_PROGRESS (pbar);
+  GdkRectangle area;
+
+  switch (orientation)
+    {
+    case GTK_PROGRESS_LEFT_TO_RIGHT:
+    case GTK_PROGRESS_RIGHT_TO_LEFT:
+      gtk_progress_bar_get_activity (pbar, orientation, &area.x, &area.width);
+      area.y = widget->style->ythickness;
+      area.height = widget->allocation.height - 2 * widget->style->ythickness;
+      break;
+
+    case GTK_PROGRESS_TOP_TO_BOTTOM:
+    case GTK_PROGRESS_BOTTOM_TO_TOP:
+      gtk_progress_bar_get_activity (pbar, orientation, &area.y, &area.height);
+      area.x = widget->style->xthickness;
+      area.width = widget->allocation.width - 2 * widget->style->xthickness;
       break;
 
     default:
@@ -615,8 +669,8 @@ gtk_progress_bar_paint_activity (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 progress->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -624,8 +678,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
 				   gint                       amount,
 				   GtkProgressBarOrientation  orientation)
 {
+  GdkRectangle area;
   GtkWidget *widget = GTK_WIDGET (pbar);
-  gint x, y, w, h;
 
   if (amount <= 0)
     return;
@@ -634,24 +688,24 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      w = amount;
-      h = widget->allocation.height - widget->style->ythickness * 2;
-      y = widget->style->ythickness;
+      area.width = amount;
+      area.height = widget->allocation.height - widget->style->ythickness * 2;
+      area.y = widget->style->ythickness;
       
-      x = widget->style->xthickness;
+      area.x = widget->style->xthickness;
       if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	x = widget->allocation.width - amount - x;
+	area.x = widget->allocation.width - amount - area.x;
       break;
       
     case GTK_PROGRESS_TOP_TO_BOTTOM:
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      w = widget->allocation.width - widget->style->xthickness * 2;
-      h = amount;
-      x = widget->style->xthickness;
+      area.width = widget->allocation.width - widget->style->xthickness * 2;
+      area.height = amount;
+      area.x = widget->style->xthickness;
       
-      y = widget->style->ythickness;
+      area.y = widget->style->ythickness;
       if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	y = widget->allocation.height - amount - y;
+	area.y = widget->allocation.height - amount - area.y;
       break;
       
     default:
@@ -662,8 +716,8 @@ gtk_progress_bar_paint_continuous (GtkProgressBar            *pbar,
   gtk_paint_box (widget->style,
 		 GTK_PROGRESS (pbar)->offscreen_pixmap,
 		 GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		 NULL, widget, "bar",
-		 x, y, w, h);
+		 &area, widget, "bar",
+		 area.x, area.y, area.width, area.height);
 }
 
 static void
@@ -675,7 +729,8 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 
   for (i = 0; i <= pbar->in_block; i++)
     {
-      gint x, y, w, h, space;
+      GdkRectangle area;
+      gint space;
 
       switch (orientation)
 	{
@@ -683,26 +738,26 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
 	case GTK_PROGRESS_RIGHT_TO_LEFT:
 	  space = widget->allocation.width - 2 * widget->style->xthickness;
 	  
-	  x = widget->style->xthickness + (i * space) / pbar->blocks;
-	  y = widget->style->ythickness;
-	  w = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - x;
-	  h = widget->allocation.height - 2 * widget->style->ythickness;
+	  area.x = widget->style->xthickness + (i * space) / pbar->blocks;
+	  area.y = widget->style->ythickness;
+	  area.width = widget->style->xthickness + ((i + 1) * space) / pbar->blocks - area.x;
+	  area.height = widget->allocation.height - 2 * widget->style->ythickness;
 
 	  if (orientation == GTK_PROGRESS_RIGHT_TO_LEFT)
-	    x = widget->allocation.width - w - x;
+	    area.x = widget->allocation.width - area.width - area.x;
 	  break;
 	  
 	case GTK_PROGRESS_TOP_TO_BOTTOM:
 	case GTK_PROGRESS_BOTTOM_TO_TOP:
 	  space = widget->allocation.height - 2 * widget->style->ythickness;
 	  
-	  x = widget->style->xthickness;
-	  y = widget->style->ythickness + (i * space) / pbar->blocks;
-	  w = widget->allocation.width - 2 * widget->style->xthickness;
-	  h = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - y;
+	  area.x = widget->style->xthickness;
+	  area.y = widget->style->ythickness + (i * space) / pbar->blocks;
+	  area.width = widget->allocation.width - 2 * widget->style->xthickness;
+	  area.height = widget->style->ythickness + ((i + 1) * space) / pbar->blocks - area.y;
 	  
 	  if (orientation == GTK_PROGRESS_BOTTOM_TO_TOP)
-	    y = widget->allocation.height - h - y;
+	    area.y = widget->allocation.height - area.height - area.y;
 	  break;
 
 	default:
@@ -713,76 +768,121 @@ gtk_progress_bar_paint_discrete (GtkProgressBar            *pbar,
       gtk_paint_box (widget->style,
 		     GTK_PROGRESS (pbar)->offscreen_pixmap,
 		     GTK_STATE_PRELIGHT, GTK_SHADOW_OUT,
-		     NULL, widget, "bar",
-		     x, y, w, h);
+		     &area, widget, "bar",
+		     area.x, area.y, area.width, area.height);
     }
 }
 
 static void
 gtk_progress_bar_paint_text (GtkProgressBar            *pbar,
+			     gint                       offset,
 			     gint			amount,
 			     GtkProgressBarOrientation  orientation)
 {
   GtkProgress *progress = GTK_PROGRESS (pbar);
   GtkWidget *widget = GTK_WIDGET (pbar);
-  
   gint x;
   gint y;
   gchar *buf;
   GdkRectangle rect;
   PangoLayout *layout;
   PangoRectangle logical_rect;
-  GdkRectangle prelight_clip, normal_clip;
-  
+  GdkRectangle prelight_clip, start_clip, end_clip;
+  gfloat text_xalign = progress->x_align;
+  gfloat text_yalign = progress->y_align;
+
+  if (gtk_widget_get_direction (widget) != GTK_TEXT_DIR_LTR)
+    text_xalign = 1.0 - text_xalign;
+
   buf = gtk_progress_get_current_text (progress);
   
   layout = gtk_widget_create_pango_layout (widget, buf);
+  pango_layout_set_ellipsize (layout, pbar->ellipsize);
+  if (pbar->ellipsize)
+    pango_layout_set_width (layout, widget->allocation.width * PANGO_SCALE);
+
   pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
   
-  x = widget->style->xthickness + 1 +
-    (widget->allocation.width - 2 * widget->style->xthickness -
-     2 - logical_rect.width)
-    * progress->x_align; 
+  x = widget->style->xthickness + 1 + text_xalign *
+      (widget->allocation.width - 2 * widget->style->xthickness -
+       2 - logical_rect.width);
 
-  y = widget->style->ythickness + 1 +
-    (widget->allocation.height - 2 * widget->style->ythickness -
-     2 - logical_rect.height)
-    * progress->y_align;
+  y = widget->style->ythickness + 1 + text_yalign *
+      (widget->allocation.height - 2 * widget->style->ythickness -
+       2 - logical_rect.height);
 
   rect.x = widget->style->xthickness;
   rect.y = widget->style->ythickness;
   rect.width = widget->allocation.width - 2 * widget->style->xthickness;
   rect.height = widget->allocation.height - 2 * widget->style->ythickness;
 
-  prelight_clip = normal_clip = rect;
+  prelight_clip = start_clip = end_clip = rect;
 
   switch (orientation)
     {
     case GTK_PROGRESS_LEFT_TO_RIGHT:
+      if (offset != -1)
+	prelight_clip.x = offset;
       prelight_clip.width = amount;
-      normal_clip.x += amount;
-      normal_clip.width -= amount;
+      start_clip.width = prelight_clip.x - start_clip.x;
+      end_clip.x = start_clip.x + start_clip.width + prelight_clip.width;
+      end_clip.width -= prelight_clip.width + start_clip.width;
       break;
       
     case GTK_PROGRESS_RIGHT_TO_LEFT:
-      normal_clip.width -= amount;
-      prelight_clip.x += normal_clip.width;
-      prelight_clip.width -= normal_clip.width;
+      if (offset != -1)
+	prelight_clip.x = offset;
+      else
+	prelight_clip.x = rect.x + rect.width - amount;
+      prelight_clip.width = amount;
+      start_clip.width = prelight_clip.x - start_clip.x;
+      end_clip.x = start_clip.x + start_clip.width + prelight_clip.width;
+      end_clip.width -= prelight_clip.width + start_clip.width;
       break;
        
     case GTK_PROGRESS_TOP_TO_BOTTOM:
+      if (offset != -1)
+	prelight_clip.y = offset;
       prelight_clip.height = amount;
-      normal_clip.y += amount;
-      normal_clip.height -= amount;
+      start_clip.height = prelight_clip.y - start_clip.y;
+      end_clip.y = start_clip.y + start_clip.height + prelight_clip.height;
+      end_clip.height -= prelight_clip.height + start_clip.height;
       break;
       
     case GTK_PROGRESS_BOTTOM_TO_TOP:
-      normal_clip.height -= amount;
-      prelight_clip.y += normal_clip.height;
-      prelight_clip.height -= normal_clip.height;
+      if (offset != -1)
+	prelight_clip.y = offset;
+      else
+	prelight_clip.y = rect.y + rect.height - amount;
+      prelight_clip.height = amount;
+      start_clip.height = prelight_clip.y - start_clip.y;
+      end_clip.y = start_clip.y + start_clip.height + prelight_clip.height;
+      end_clip.height -= prelight_clip.height + start_clip.height;
       break;
     }
-  
+
+  if (start_clip.width > 0 && start_clip.height > 0)
+    gtk_paint_layout (widget->style,
+		      progress->offscreen_pixmap,
+		      GTK_STATE_NORMAL,
+		      FALSE,
+		      &start_clip,
+		      widget,
+		      "progressbar",
+		      x, y,
+		      layout);
+
+  if (end_clip.width > 0 && end_clip.height > 0)
+    gtk_paint_layout (widget->style,
+		      progress->offscreen_pixmap,
+		      GTK_STATE_NORMAL,
+		      FALSE,
+		      &end_clip,
+		      widget,
+		      "progressbar",
+		      x, y,
+		      layout);
+
   gtk_paint_layout (widget->style,
 		    progress->offscreen_pixmap,
 		    GTK_STATE_PRELIGHT,
@@ -792,16 +892,6 @@ gtk_progress_bar_paint_text (GtkProgressBar            *pbar,
 		    "progressbar",
 		    x, y,
 		    layout);
-  
-  gtk_paint_layout (widget->style,
-		    progress->offscreen_pixmap,
-		    GTK_STATE_NORMAL,
-		    FALSE,
-		    &normal_clip,
-		    widget,
-		    "progressbar",
-		    x, y,
-		    layout);
 
   g_object_unref (layout);
   g_free (buf);
@@ -842,6 +932,15 @@ gtk_progress_bar_paint (GtkProgress *progress)
       if (progress->activity_mode)
 	{
 	  gtk_progress_bar_paint_activity (pbar, orientation);
+
+	  if (GTK_PROGRESS (pbar)->show_text)
+	    {
+	      gint offset;
+	      gint amount;
+
+	      gtk_progress_bar_get_activity (pbar, orientation, &offset, &amount);
+	      gtk_progress_bar_paint_text (pbar, offset, amount, orientation);
+	    }
 	}
       else
 	{
@@ -862,11 +961,13 @@ gtk_progress_bar_paint (GtkProgress *progress)
 	      gtk_progress_bar_paint_continuous (pbar, amount, orientation);
 
 	      if (GTK_PROGRESS (pbar)->show_text)
-		gtk_progress_bar_paint_text (pbar, amount, orientation);
+		gtk_progress_bar_paint_text (pbar, -1, amount, orientation);
 	    }
 	  else
 	    gtk_progress_bar_paint_discrete (pbar, orientation);
 	}
+
+      pbar->dirty = FALSE;
     }
 }
 
@@ -883,7 +984,7 @@ gtk_progress_bar_set_bar_style_internal (GtkProgressBar     *pbar,
       if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
 	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "bar_style");
+      g_object_notify (G_OBJECT (pbar), "bar-style");
     }
 }
 
@@ -901,7 +1002,7 @@ gtk_progress_bar_set_discrete_blocks_internal (GtkProgressBar *pbar,
       if (GTK_WIDGET_DRAWABLE (GTK_WIDGET (pbar)))
 	gtk_widget_queue_resize (GTK_WIDGET (pbar));
 
-      g_object_notify (G_OBJECT (pbar), "discrete_blocks");
+      g_object_notify (G_OBJECT (pbar), "discrete-blocks");
     }
 }
 
@@ -914,7 +1015,7 @@ gtk_progress_bar_set_activity_step_internal (GtkProgressBar *pbar,
   if (pbar->activity_step != step)
     {
       pbar->activity_step = step;
-      g_object_notify (G_OBJECT (pbar), "activity_step");
+      g_object_notify (G_OBJECT (pbar), "activity-step");
     }
 }
 
@@ -928,7 +1029,7 @@ gtk_progress_bar_set_activity_blocks_internal (GtkProgressBar *pbar,
   if (pbar->activity_blocks != blocks)
     {
       pbar->activity_blocks = blocks;
-      g_object_notify (G_OBJECT (pbar), "activity_blocks");
+      g_object_notify (G_OBJECT (pbar), "activity-blocks");
     }
 }
 
@@ -987,7 +1088,7 @@ gtk_progress_bar_pulse (GtkProgressBar *pbar)
 /**
  * gtk_progress_bar_set_text:
  * @pbar: a #GtkProgressBar
- * @text: a UTF-8 string
+ * @text: a UTF-8 string, or %NULL 
  * 
  * Causes the given @text to appear superimposed on the progress bar.
  **/
@@ -1025,7 +1126,7 @@ gtk_progress_bar_set_pulse_step   (GtkProgressBar *pbar,
   
   pbar->pulse_fraction = fraction;
 
-  g_object_notify (G_OBJECT (pbar), "pulse_step");
+  g_object_notify (G_OBJECT (pbar), "pulse-step");
 }
 
 void
@@ -1174,3 +1275,52 @@ gtk_progress_bar_set_activity_blocks (GtkProgressBar *pbar,
 
   gtk_progress_bar_set_activity_blocks_internal (pbar, blocks);
 }
+
+/**
+ * gtk_progress_bar_set_ellipsize:
+ * @pbar: a #GtkProgressBar
+ * @mode: a #PangoEllipsizeMode
+ *
+ * Sets the mode used to ellipsize (add an ellipsis: "...") the text 
+ * if there is not enough space to render the entire string.
+ *
+ * Since: 2.6
+ **/
+void
+gtk_progress_bar_set_ellipsize (GtkProgressBar     *pbar,
+				PangoEllipsizeMode  mode)
+{
+  g_return_if_fail (GTK_IS_PROGRESS_BAR (pbar));
+  g_return_if_fail (mode >= PANGO_ELLIPSIZE_NONE && 
+		    mode <= PANGO_ELLIPSIZE_END);
+  
+  if ((PangoEllipsizeMode)pbar->ellipsize != mode)
+    {
+      pbar->ellipsize = mode;
+
+      g_object_notify (G_OBJECT (pbar), "ellipsize");
+      gtk_widget_queue_resize (GTK_WIDGET (pbar));
+    }
+}
+
+/**
+ * gtk_progress_bar_get_ellipsize:
+ * @pbar: a #GtkProgressBar
+ *
+ * Returns the ellipsizing position of the progressbar. 
+ * See gtk_progress_bar_set_ellipsize().
+ *
+ * Return value: #PangoEllipsizeMode
+ *
+ * Since: 2.6
+ **/
+PangoEllipsizeMode 
+gtk_progress_bar_get_ellipsize (GtkProgressBar *pbar)
+{
+  g_return_val_if_fail (GTK_IS_PROGRESS_BAR (pbar), PANGO_ELLIPSIZE_NONE);
+
+  return pbar->ellipsize;
+}
+
+#define __GTK_PROGRESS_BAR_C__
+#include "gtkaliasdef.c"
